#include <stdio.h>
#include <stdlib.h>

int min( int a, int b ) { return a < b ? a : b; }
int max( int a, int b ) { return a > b ? a : b; }

// ------------------------------ Grupa A --------------------------------
char *grupaA( char **A, int n, int k )
{
	// sporije rjesenje: O(n^2 * k) - vidi grupaC za brze rjesenje
	int red, stup, stup1, brRazbijenih = 0, zadnja = 0;
	char *razbijene;
	
	for( red = 0; red < n; ++red )
		for( stup = max(n-k, 0); stup < n; ++stup )
			if( A[ red ][ stup ] != '.' )
				++brRazbijenih;
				
	razbijene = (char *) malloc( (brRazbijenih+1) * sizeof(char) );
	
	for( red = 0; red < n; ++red )
		for( stup = n-1; stup >= 0; --stup )
			if( A[ red ][ stup ] != '.' )
			{
				if( stup + k < n ) 
					A[ red ][ stup+k ] = A[ red ][ stup ];
				else
					razbijene[ zadnja++ ] = A[ red ][ stup ];

				for( stup1 = stup; stup1 < min(n, stup+k); ++stup1 )
					A[ red ][ stup1 ] = '-';
			}
	
	razbijene[ brRazbijenih ] = '\0';
	return razbijene;
}

// ------------------------------ Grupa B --------------------------------
int *grupaB( int **A, int n, int k, int *brRazbijenih )
{
	// sporije rjesenje: O(n^2 * k) - vidi grupaC za brze rjesenje
	int red, stup, stup1, zadnja = 0, *razbijene;

	*brRazbijenih = 0;	
	for( red = 0; red < n; ++red )
		for( stup = 0; stup < min(k, n); ++stup )
			if( A[ red ][ stup ] != 0 )
				++(*brRazbijenih);
				
	razbijene = (int *) malloc( (*brRazbijenih) * sizeof(int) );
	
	for( red = 0; red < n; ++red )
		for( stup = 0; stup < n; ++stup )
			if( A[ red ][ stup ] != 0 )
			{
				if( stup >= k ) 
					A[ red ][ stup-k ] = A[ red ][ stup ];
				else
					razbijene[ zadnja++ ] = A[ red ][ stup ];

				for( stup1 = max(0, stup-k+1); stup1 <= stup; ++stup1 )
					A[ red ][ stup1 ] = -1;
			}
	
	return razbijene;
}

// ------------------------------ Grupa C --------------------------------
char *grupaC( char **A, int n, int k )
{
	// brze rjesenje: O(n^2)
	int red, stup, brRazbijenih = 0, zadnja = 0;
	char *razbijene;
	
	k = min( n, k );
	for( red = n-k; red < n; ++red )
		for( stup = 0; stup < n; ++stup )
			if( A[ red ][ stup ] != '.' )
				++brRazbijenih;
				
	razbijene = (char *) malloc( (brRazbijenih+1) * sizeof(char) );
	
	for( stup = 0; stup < n; ++stup )
	{
		int redGledam = n-1, redPisem = redGledam+k, kolikoJosPomaka = -1;
		
		while( redPisem >= 0 )
		{
			char znak = ( redGledam >= 0 ) ? A[ redGledam ][ stup ] : '.';
			if( znak != '.' ) kolikoJosPomaka = k;
			
			if( redPisem < n )
				A[ redPisem ][ stup ] = ( kolikoJosPomaka == k || kolikoJosPomaka < 0 ) ? znak : '|';
			else if( znak != '.' )
				razbijene[ zadnja++ ] = znak;
			
			--redGledam; --redPisem; --kolikoJosPomaka;
		}
	}	
	
	razbijene[ brRazbijenih ] = '\0';
	return razbijene;
}

// ------------------------------ Grupa D --------------------------------
int *grupaD( int **A, int n, int k, int *brRazbijenih )
{
	// sporije rjesenje: O(n^2 * k) - vidi grupaC za brze rjesenje
	int red, stup, red1, zadnja = 0, *razbijene;
	
	*brRazbijenih = 0;	
	for( red = 0; red < min(k, n); ++red )
		for( stup = 0; stup < n; ++stup )
			if( A[ red ][ stup ] != -1 )
				++(*brRazbijenih);
				
	razbijene = (int *) malloc( (*brRazbijenih) * sizeof(int) );
	
	for( red = 0; red < n; ++red )
		for( stup = 0; stup < n; ++stup )
			if( A[ red ][ stup ] != -1 )
			{
				if( red >= k ) 
					A[ red-k ][ stup ] = A[ red ][ stup ];
				else
					razbijene[ zadnja++ ] = A[ red ][ stup ];

				for( red1 = max(0, red-k+1); red1 <= red; ++red1 )
					A[ red1 ][ stup ] = 0;
			}
	
	return razbijene;
}

int main( void ) // ne treba pisati na kolokviju, ovdje je samo za provjeru
{
	char temp[4][5] = { 
		"...A", 
		"AB..",
		"C.DF",
		".B.."
	};
		
	char *A[ 4 ] = { temp[0], temp[1], temp[2], temp[3] };
	char *ret = grupaC( A, 4, 2 );
	int red, stup;
	
	printf( "Razbijene: %s\n", ret );
	for( red = 0; red < 4; ++red, printf( "\n" ) )
		for( stup = 0; stup < 4; ++stup )
			printf( "%c", A[ red ][ stup ] );

	free( ret );
	
	return 0;
}
